#ifndef __HotStateAnalyzer_h__
#define __HotStateAnalyzer_h__

#include "TemperatureToken.h"

class HotStateAnalyzer
{
public:

    void analyze( ThreadStateGraph &graph, TemperatureToken &token )
    {
        storeInput( graph, token );
        findHotStates();
        classifyStates();
    }

    Vector<String> getEscapables()
    {
        return _hotEscapables;
    }

    Vector<String> getTraps()
    {
        return _hotTraps;
    }

private:
    TemperatureToken *_token;
    ThreadStateGraph *_graph;
    Vector<String> _hotStates;
    Vector<String> _hotEscapables;
    Vector<String> _hotTraps;
    Vector<String> _checked;
    Vector<String> _toCheck;

    void resetDfs( const String &root )
    {
        _checked.clear();
        _toCheck.clear();

        _toCheck.append( root );
        _checked.append( root );
    }

    void storeInput( ThreadStateGraph &graph, TemperatureToken &token )
    {
        _graph = &graph;
        _token = &token;
    }

    void findHotStates()
    {
        Map<String, ThreadState>::iterator iterator;

        for ( iterator = _graph->_graph.begin(); iterator != _graph->_graph.end(); ++iterator )
        {
            if ( hotState( iterator->first ) )
            {
                _hotStates.append( iterator->second._label );
            }
        }
    }

    bool hotState( const String &label ) const
    {
        return _token->hot( _graph->_graph[label] );
    }

    bool coldState( const String &label ) const
    {
        return !hotState( label );
    }

    void dumpHotStates()
    {
        printf( "Hot states are:\n" );
        for ( unsigned i = 0; i < _hotStates.size(); ++i )
        {
            printf( "%s\n", _hotStates[i].ascii() );
        }
    }

    void classifyStates()
    {
        for ( unsigned i = 0; i < _hotStates.size(); ++i )
        {
            classifyHotState( _hotStates[i] );
        }
    }

    void classifyHotState( const String &state )
    {
        if ( haveColdSuccessors( state ) )
        {
            _hotEscapables.append( state );
        }
        else
        {
            _hotTraps.append( state );
        }
    }

    bool haveColdSuccessors( const String &root )
    {
        resetDfs( root );

        while ( !_toCheck.empty() )
        {
            String currentState = _toCheck.popFirst();
            Vector<String> successors = _graph->_graph[currentState].allSuccessors();

            for ( unsigned i = 0; i < successors.size(); ++i )
            {
                String successor = successors[i];

                if ( coldState( successor ) )
                {
                    return true;
                }

                if ( unvisitedState( successor ) )
                {
                    _checked.append( successor );
                    _toCheck.append( successor );
                }
            }
        }

        return false;
    }

    bool unvisitedState( const String &label )
    {
        return !_checked.exists( label );
    }
};

#endif // __HotStateAnalyzer_h__

//
// Local Variables:
// compile-command: "make -C ../.. "
// tags-file-name: "../../TAGS"
// c-basic-offset: 4
// End:
//
