#ifndef __Measurer_h__
#define __Measurer_h__

#include "BThread.h"
#include "IEvent.h"
#include "String.h"
#include "T/Truth.h"

class Measurer : public BThread
{
public:
    Measurer() : BThread( "Measurer" )
    {
    }

    void entryPoint()
    {
        while ( T::alwaysTrue() )
        {
            waitForMaintenance();

            checkTemperature();
            issueTemperatureNotice();
            waitForTemperatureResponse();

            checkHumidity();
            issueHumidityNotice();
            waitForHumidityResponse();
        }
    }

private:
    enum TemperatureStatus {
        TEMPERATURE_LOW = 0,
        TEMPERATURE_HIGH = 1,

        TOTAL_TEMPERATURE_OPTIONS = 2,
    };

    enum HumidityStatus {
        HUMIDITY_LOW = 0,
        HUMIDITY_HIGH = 1,

        TOTAL_HUMIDITY_OPTIONS = 2,
    };

    Vector<Event> _requested;
    Vector<Event> _watched;
    Vector<Event> _blocked;

    TemperatureStatus _temperatureStatus;
    HumidityStatus _humidityStatus;

    void clearAll()
    {
        _requested.clear();
        _watched.clear();
        _blocked.clear();
    }

    void waitForMaintenance()
    {
        clearAll();
        _watched.append( IEvent::INITIATE_MAINTENANCE );
        bSync( _requested, _watched, _blocked, "Waiting for maintenance" );
    }

    void checkTemperature()
    {
        _temperatureStatus = (TemperatureStatus)( rand() % TOTAL_TEMPERATURE_OPTIONS );
        temperatureMeasurementDelay();
    }

    void issueTemperatureNotice()
    {
        clearAll();

        _blocked.append( IEvent::PRODUCT_PROVIDED );
        _watched.append( IEvent::TEMPERATURE_LOW );
        _watched.append( IEvent::TEMPERATURE_HIGH );

        switch ( _temperatureStatus )
        {
        case TEMPERATURE_LOW:
            _requested.append( IEvent::TEMPERATURE_LOW );
            bSync( _requested, _watched, _blocked, "Issuing temperature notice -- low" );
            break;
        case TEMPERATURE_HIGH:
            _requested.append( IEvent::TEMPERATURE_HIGH );
            bSync( _requested, _watched, _blocked, "Issuing temperature notice -- high" );
            break;
        default:
            break;
        }
    }

    void waitForTemperatureResponse()
    {
        clearAll();
        _watched.append( IEvent::TEMPERATURE_ADJUSTED );
        _blocked.append( IEvent::PRODUCT_PROVIDED );
        bSync( _requested, _watched, _blocked, "Wait for temperature response" );
    }

    void checkHumidity()
    {
        _humidityStatus = (HumidityStatus)( rand() % TOTAL_HUMIDITY_OPTIONS );
        humidityMeasurementDelay();
    }

    void issueHumidityNotice()
    {
        clearAll();

        _watched.append( IEvent::HUMIDITY_LOW );
        _watched.append( IEvent::HUMIDITY_HIGH );

        switch ( _humidityStatus )
        {
        case HUMIDITY_LOW:
            _requested.append( IEvent::HUMIDITY_LOW );
            bSync( _requested, _watched, _blocked, "Issuing humidity notice -- low" );
            break;
        case HUMIDITY_HIGH:
            _requested.append( IEvent::HUMIDITY_HIGH );
            bSync( _requested, _watched, _blocked, "Issuing humidity notice -- high" );
            break;
        default:
            break;
        }
    }

    void waitForHumidityResponse()
    {
        clearAll();
        _watched.append( IEvent::HUMIDITY_ADJUSTED );
        bSync( _requested, _watched, _blocked, "Wait for humidity response" );
    }

    void temperatureMeasurementDelay()
    {
        enum {
            SECONDS_TO_ADJUST_TEMPERATURE = 3,
        };

        sleep( SECONDS_TO_ADJUST_TEMPERATURE );
    }

    void humidityMeasurementDelay()
    {
        enum {
            SECONDS_TO_ADJUST_HUMIDITY = 3,
        };

        sleep( SECONDS_TO_ADJUST_HUMIDITY );
    }
};

#endif // __Measurer_h__

//
// Local Variables:
// compile-command: "make -C .. "
// tags-file-name: "../TAGS"
// c-basic-offset: 4
// End:
//
